/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.lcs.client.internal.advisor;

import com.liferay.lcs.client.internal.exception.InitializationException;
import com.liferay.portal.kernel.log.Log;
import com.liferay.portal.kernel.log.LogFactoryUtil;
import com.liferay.portal.kernel.util.Digester;
import com.liferay.portal.kernel.util.DigesterUtil;
import com.liferay.portal.kernel.util.FileUtil;
import com.liferay.portal.kernel.util.PropsKeys;
import com.liferay.portal.kernel.util.PropsUtil;

import java.io.File;
import java.io.IOException;

import java.util.Arrays;
import java.util.UUID;

import org.osgi.service.component.annotations.Component;

/**
 * @author Ivica Cardic
 * @author Igor Beslic
 */
@Component(service = LCSKeyAdvisor.class)
public class LCSKeyAdvisor {

	public void clearCache() {
		_key = null;
	}

	public synchronized String getKey() {
		if (_key != null) {
			return _key;
		}

		_readKey();

		return _key;
	}

	public synchronized String getTemporaryKey() {
		if (_temporaryKey != null) {
			return _temporaryKey;
		}

		UUID uuid = UUID.randomUUID();

		_temporaryKey = _LCS_KEY_TEMPORARY_PREFIX + uuid.toString();

		return _temporaryKey;
	}

	public synchronized void updateKey(String key) {
		String lcsKeyFilePath = getLCSKeyFilePath();

		try {
			File lcsServerIdFile = new File(lcsKeyFilePath);

			FileUtil.write(lcsServerIdFile, key.getBytes());

			_key = key;
			_temporaryKey = null;

			if (_log.isInfoEnabled()) {
				_log.info("LCS server key was updated. New value: " + _key);
			}
		}
		catch (IOException ioe) {
			throw new InitializationException.FileSystemAccessException(
				lcsKeyFilePath, ioe);
		}
	}

	@Override
	protected void finalize() throws Throwable {
		super.finalize();

		if (_log.isTraceEnabled()) {
			_log.trace("Finalized " + this);
		}
	}

	protected String getLCSKeyFilePath() {
		return PropsUtil.get(PropsKeys.LIFERAY_HOME) +
			_LCS_KEY_FILE_RELATIVE_PATH;
	}

	private void _readKey() {
		String lcsKeyFilePath = getLCSKeyFilePath();

		try {
			File lcsServerIdFile = new File(lcsKeyFilePath);

			if (!lcsServerIdFile.exists()) {
				if (_log.isDebugEnabled()) {
					_log.debug("LCS server key file does not exist");
				}

				return;
			}

			byte[] keyBytes = FileUtil.getBytes(lcsServerIdFile);

			if (Arrays.binarySearch(keyBytes, (byte)45) >= 0) {
				_key = new String(keyBytes);
			}
			else {
				_key = Arrays.toString(keyBytes);

				_key = DigesterUtil.digestHex(Digester.MD5, _key);

				if (_log.isTraceEnabled()) {
					_log.trace(
						"LCS server key is in legacy format, generated by " +
							"LCS Client 4");
				}
			}

			if (_log.isTraceEnabled()) {
				_log.trace("LCS server key was read from file system: " + _key);
			}
		}
		catch (IOException ioe) {
			throw new InitializationException.FileSystemAccessException(
				lcsKeyFilePath, ioe);
		}
	}

	private static final String _LCS_KEY_FILE_RELATIVE_PATH =
		"/data/license/server/lcsServerId";

	private static final String _LCS_KEY_TEMPORARY_PREFIX = "TEMP-KEY-";

	private static final Log _log = LogFactoryUtil.getLog(LCSKeyAdvisor.class);

	private String _key;
	private String _temporaryKey;

}